@@ -1,39 +1,21 @@
-/* Copyright (C) 2000 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2000 artofcode LLC.  All rights reserved.
 
-   This program may be distributed as part of AFPL Ghostscript, under the
-   terms of the Aladdin Free Public License (the "License").
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; either version 2 of the License, or (at your
+  option) any later version.
 
-   AFPL Ghostscript is distributed with NO WARRANTY OF ANY KIND.  No
-   author or distributor accepts any responsibility for the consequences of
-   using it, or for whether it serves any particular purpose or works at
-   all, unless he or she says so in writing.  Refer to the License for full
-   details.
+  This program is distributed in the hope that it will be useful, but
+  WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+  General Public License for more details.
 
-   Every copy of AFPL Ghostscript must include a copy of the License,
-   normally in a plain ASCII text file named PUBLIC.  The License grants you
-   the right to copy, modify and redistribute AFPL Ghostscript, but only
-   under certain conditions described in the License.  Among other things,
-   the License requires that the copyright notice and this notice be
-   preserved on all copies.
-
-   This program may also be distributed and/or modified under the terms of
-   the GNU General Public License as published by the Free Software
-   Foundation (the "GPL"); either version 2 of the GPL, or (at your option)
-   any later version.
-
-   When distributed under the terms of the GPL, this program is distributed
-   in the hope that it will be useful, but WITHOUT ANY WARRANTY; without
-   even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
-   PURPOSE.  See the GPL for more details.
-
-   If this program is being distributed under the terms of the GPL, you
-   should have received a copy of the GPL along with this program, normally
-   in a plain ASCII text file named COPYING; if not, write to the Free
-   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111
-   U.S.A.
+  You should have received a copy of the GNU General Public License along
+  with this program; if not, write to the Free Software Foundation, Inc.,
+  59 Temple Place, Suite 330, Boston, MA, 02111-1307.
  */
 
-/*$Id: gdevdljm.h,v 1.4 2000/09/19 19:00:12 lpd Exp $ */
+/*$Id: gdevdljm.h,v 1.4.8.1 2003/01/17 00:49:00 giles Exp $ */
 /* Interface to generic monochrome H-P DeskJet/LaserJet driver */
 
 #ifndef gdevdljm_INCLUDED
