@@ -1,21 +1,23 @@
 /* Copyright (C) 2001, Ghostgum Software Pty Ltd.  All rights reserved.
 
-   This software is provided AS-IS with no warranty, either express or
-   implied.
-   
-   This software is distributed under license and may not be copied,
-   modified or distributed except as expressly authorized under the terms
-   of the license contained in the file LICENSE in this distribution.
-   
-   For more information about licensing, please refer to
-   http://www.ghostscript.com/licensing/. For information on
-   commercial licensing, go to http://www.artifex.com/licensing/ or
-   contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; either version 2 of the License, or (at your
+  option) any later version.
+
+  This program is distributed in the hope that it will be useful, but
+  WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+  General Public License for more details.
+
+  You should have received a copy of the GNU General Public License along
+  with this program; if not, write to the Free Software Foundation, Inc.,
+  59 Temple Place, Suite 330, Boston, MA, 02111-1307.
+
  */
 
 /* gdevdsp.c */
-/* $Id: gdevdsp.c,v 1.5.2.3 2002/04/10 09:33:25 giles Exp $ */
+/* $Id: gdevdsp.c,v 1.5.2.5.2.3 2003/05/01 10:06:26 ghostgum Exp $ */
 
 /*
  * DLL based display device driver.
@@ -76,6 +78,8 @@ private dev_proc_map_rgb_color(display_map_rgb_color_rgb);
 private dev_proc_map_color_rgb(display_map_color_rgb_rgb);
 private dev_proc_map_rgb_color(display_map_rgb_color_bgr24);
 private dev_proc_map_color_rgb(display_map_color_rgb_bgr24);
+private dev_proc_map_rgb_color(display_map_rgb_color_cmyk1);
+private dev_proc_map_rgb_color(display_map_rgb_color_cmyk8);
 
 private dev_proc_fill_rectangle(display_fill_rectangle);
 private dev_proc_copy_mono(display_copy_mono);
@@ -146,15 +150,6 @@ ENUM_PTRS_WITH(display_enum_ptrs, gx_device_display *ddev) return 0;
 	if (ddev->mdev) {
 	    return ENUM_OBJ(gx_device_enum_ptr((gx_device *)ddev->mdev));
 	}
-	return 0;	/* if mdev is NULL, then pBitmap will be also */
-    case 1: 
-        if (ddev->callback && 
-	    !ddev->callback->display_memalloc && 
-	    !ddev->callback->display_memfree &&
-	    ddev->pBitmap) {
-	    /* we allocated the bitmap */
-	    return ENUM_OBJ(ddev->pBitmap);
-	}
 	return 0;
 ENUM_PTRS_END
 
@@ -316,10 +311,6 @@ private gx_color_index
 display_map_rgb_color_device4(gx_device * dev, 
     gx_color_value r, gx_color_value g, gx_color_value b)
 {
-    gx_device_display *ddev = (gx_device_display *) dev;
-    if ((r == g) && (g == b) && (r >= gx_max_color_value / 3 * 2 - 1)
-	&& (r < gx_max_color_value / 4 * 3))
-	return ((gx_color_index) 8);	/* light gray */
     return pc_4bit_map_rgb_color(dev, r, g, b);
 }
 
@@ -328,10 +319,7 @@ private int
 display_map_color_rgb_device4(gx_device * dev, gx_color_index color,
 		 gx_color_value prgb[3])
 {
-    if (color == 8)	/* VGA light grey */
-	prgb[0] = prgb[1] = prgb[2] = (gx_max_color_value / 4 * 3);
-    else
-	pc_4bit_map_color_rgb(dev, color, prgb);
+    pc_4bit_map_color_rgb(dev, color, prgb);
     return 0;
 }
 
@@ -455,7 +443,7 @@ display_map_color_rgb_device16(gx_device * dev, gx_color_index color,
 	    value = ((color << 3) & 0x18) +  ((color >> 13) & 0x7);
 	    prgb[1] = ((value << 11) + (value << 6) + (value << 1) + 
 	    (value >> 4)) >> (16 - gx_color_value_bits);
-	    value = color & 0x1f;
+	    value = (color >> 8) & 0x1f;
 	    prgb[2] = ((value << 11) + (value << 6) + (value << 1) + 
 		(value >> 4)) >> (16 - gx_color_value_bits);
 	}
@@ -467,7 +455,7 @@ display_map_color_rgb_device16(gx_device * dev, gx_color_index color,
 	    value = ((color << 3) & 0x38) +  ((color >> 13) & 0x7);
 	    prgb[1] = ((value << 10) + (value << 4) + (value >> 2))
 		      >> (16 - gx_color_value_bits);
-	    value = color & 0x1f;
+	    value = (color >> 8) & 0x1f;
 	    prgb[2] = ((value << 11) + (value << 6) + (value << 1) + 
 		(value >> 4)) >> (16 - gx_color_value_bits);
 	}
@@ -548,7 +536,7 @@ display_map_color_rgb_rgb(gx_device * dev, gx_color_index color,
 	case DISPLAY_UNUSED_FIRST:
 	    if ((ddev->nFormat & DISPLAY_ENDIAN_MASK) == DISPLAY_BIGENDIAN) {
 		/* xRGB */
-		prgb[0] = ((color) & 2*color_mask) * 
+		prgb[0] = ((color >> 2*bits_per_color) & color_mask) * 
 			(ulong) gx_max_color_value / color_mask;
 		prgb[1] = ((color >> bits_per_color)   & color_mask) * 
 			(ulong) gx_max_color_value / color_mask;
@@ -610,6 +598,48 @@ display_map_color_rgb_bgr24(gx_device * dev, gx_color_index color,
     return 0;
 }
 
+/* 8-bit CMYK
+ * This is currently used for anti-aliasing. A better solution would be to 
+ * replace gx_default_copy_alpha with a version that works in CMYK, not RGB.
+ */
+/* Map a r-g-b color to a color code */
+private gx_color_index
+display_map_rgb_color_cmyk8(gx_device * dev, gx_color_value r, 
+		gx_color_value g, gx_color_value b)
+{
+    gx_color_value c, m, y, k;
+    c = gx_max_color_value - r;
+    m = gx_max_color_value - g;
+    y = gx_max_color_value - b;
+    k = min(c, min(m,y));
+    if (k > 0) {
+	c -= k;
+	m -= k;
+	y -= k;
+    }
+    return cmyk_8bit_map_cmyk_color(dev, c, m, y, k);
+}
+
+/* 1-bit CMYK */
+/* Map a r-g-b color to a color code */
+private gx_color_index
+display_map_rgb_color_cmyk1(gx_device * dev, gx_color_value r, 
+		gx_color_value g, gx_color_value b)
+{
+    gx_color_value c, m, y, k;
+    c = gx_max_color_value - r;
+    m = gx_max_color_value - g;
+    y = gx_max_color_value - b;
+    k = min(c, min(m,y));
+    if (k > 0) {
+	c -= k;
+	m -= k;
+	y -= k;
+    }
+    return cmyk_1bit_map_cmyk_color(dev, c, m, y, k);
+}
+
+
 /* Fill a rectangle */
 private int
 display_fill_rectangle(gx_device * dev, int x, int y, int w, int h,
@@ -864,7 +894,7 @@ display_free_bitmap(gx_device_display * ddev)
 		ddev->pBitmap);
 	}
 	else {
-	    gs_free_object(gs_memory_stable(ddev->memory), 
+	    gs_free_object(&gs_memory_default,
 		ddev->pBitmap, "display_free_bitmap");
 	}
 	ddev->pBitmap = NULL;
@@ -894,12 +924,12 @@ display_alloc_bitmap(gx_device_display * ddev, gx_device * param_dev)
     mdproto = gdev_mem_device_for_bits(ddev->color_info.depth);
     if (mdproto == 0)
 	return_error(gs_error_rangecheck);
-    
-    ccode = gs_copydevice((gx_device **)&(ddev->mdev),
-			     (const gx_device *)mdproto,
-			     gs_memory_stable(ddev->memory));
-    if (ccode < 0)
-	return ccode;
+
+    ddev->mdev = gs_alloc_struct(gs_memory_stable(ddev->memory), 
+    	gx_device_memory, &st_device_memory, "display_memory_device");
+    if (ddev->mdev == 0)
+    	return_error(gs_error_VMerror);
+        
     gs_make_mem_device(ddev->mdev, mdproto, gs_memory_stable(ddev->memory), 
 	0, (gx_device *) NULL);
     gx_device_fill_in_procs((gx_device *)(ddev->mdev));
@@ -910,17 +940,22 @@ display_alloc_bitmap(gx_device_display * ddev, gx_device * param_dev)
     
     ddev->mdev->width = param_dev->width;
     ddev->mdev->height = param_dev->height;
-    ddev->ulBitmapSize = gdev_mem_bitmap_size(ddev->mdev);
+    /* Tell the memory device to allocate the line pointers separately
+     * so we can place the bitmap in special memory.
+     */
+    ddev->mdev->line_pointer_memory = ddev->mdev->memory;
+    ddev->ulBitmapSize = gdev_mem_bits_size(ddev->mdev,
+	ddev->mdev->width, ddev->mdev->height);
 
-    /* allocate bitmap */
+    /* allocate bitmap using an allocator not subject to GC */
     if (ddev->callback->display_memalloc 
 	&& ddev->callback->display_memfree) {
         ddev->pBitmap = (*ddev->callback->display_memalloc)(ddev->pHandle, 
 	    ddev, ddev->ulBitmapSize);
     }
     else {
-	ddev->pBitmap = gs_alloc_bytes_immovable(gs_memory_stable(ddev->memory),
-		(uint)ddev->ulBitmapSize, "display_alloc_bitmap");
+	ddev->pBitmap = gs_alloc_byte_array_immovable(&gs_memory_default,
+		(uint)ddev->ulBitmapSize, 1, "display_alloc_bitmap");
     }
 
     if (ddev->pBitmap == NULL) {
@@ -1089,10 +1124,12 @@ display_set_color_format(gx_device_display *ddev, int nFormat)
 	    if ((nFormat & DISPLAY_DEPTH_MASK) == DISPLAY_DEPTH_1) {
 		ddev->procs.map_cmyk_color = cmyk_1bit_map_cmyk_color;
 		ddev->procs.map_color_rgb = cmyk_1bit_map_color_rgb;
+		ddev->procs.map_rgb_color = display_map_rgb_color_cmyk1;
 	    }
 	    else if ((nFormat & DISPLAY_DEPTH_MASK) == DISPLAY_DEPTH_8) {
 		ddev->procs.map_cmyk_color = cmyk_8bit_map_cmyk_color;
 		ddev->procs.map_color_rgb = cmyk_8bit_map_color_rgb;
+		ddev->procs.map_rgb_color = display_map_rgb_color_cmyk8;
 	    }
 	    else
 		return_error(gs_error_rangecheck);
@@ -1174,7 +1211,7 @@ void
 test(int index)
 {
     char buf[1024];
-    sprintf(buf, "gs -dDisplayFormat=%d examples/colorcir.ps -c quit", test_modes[index].format);
+    sprintf(buf, "gs -dDisplayFormat=16#%x examples/colorcir.ps -c quit", test_modes[index].format);
     system(buf);
 }
 
@@ -1191,8 +1228,8 @@ int main(int argc, char *argv[])
 	}
     }
     for (i=0; i < sizeof(test_modes)/sizeof(test_mode); i++) {
-	fprintf(stdout, "%d: %s\n", test_modes[i].format,
-		test_modes[i].name);
+	fprintf(stdout, "16#%x or %d: %s\n", test_modes[i].format,
+		test_modes[i].format, test_modes[i].name);
 	if (dotest)
 	    test(i);
     }
