@@ -1,20 +1,22 @@
-/* Copyright (C) 1994-1997 artofcode LLC. All rights reserved.
+/* Copyright (C) 1994-2003 artofcode LLC. All rights reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; either version 2 of the License, or (at your
+  option) any later version.
+
+  This program is distributed in the hope that it will be useful, but
+  WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+  General Public License for more details.
+
+  You should have received a copy of the GNU General Public License along
+  with this program; if not, write to the Free Software Foundation, Inc.,
+  59 Temple Place, Suite 330, Boston, MA, 02111-1307.
+
 */
 
-/* $Id: gdevmac.c,v 1.1.4.3 2002/04/23 01:32:26 giles Exp $ */
+/* $Id: gdevmac.c,v 1.1.4.3.2.2 2003/04/12 16:37:26 giles Exp $ */
 
 #include "gdevmac.h"
 #include "gsparam.h"
@@ -149,7 +151,8 @@ mac_open(register gx_device *dev)
 	*mdev->currPicPos = 0x00ff;
 	
 	// notify the caller that a new device was opened
-	(*pgsdll_callback) (GSDLL_DEVICE, (char *)mdev, 1);
+	if (pgsdll_callback)
+		(*pgsdll_callback) (GSDLL_DEVICE, (char *)mdev, 1);
 	
 	return 0;
 }
@@ -179,7 +182,8 @@ mac_sync_output(gx_device * dev)
 	*mdev->currPicPos = 0x00ff;
 	
 	// tell the caller to sync
-	(*pgsdll_callback) (GSDLL_SYNC, (char *)mdev, 0);
+	if (pgsdll_callback)
+		(*pgsdll_callback) (GSDLL_SYNC, (char *)mdev, 0);
 	
 	return (0);
 }
@@ -200,7 +204,8 @@ mac_output_page(gx_device * dev, int copies, int flush)
 	}
 	
 	// tell the caller that the page is done
-	(*pgsdll_callback) (GSDLL_PAGE, (char *)mdev, 0);
+	if (pgsdll_callback)
+		(*pgsdll_callback) (GSDLL_PAGE, (char *)mdev, 0);
 	
 	gx_finish_output_page(dev, copies, flush);
 	
@@ -249,8 +254,9 @@ mac_close(register gx_device *dev)
 	}
 	
 	// notify the caller that the device was closed
-	// he has to dispose the PICT handle himself when he is ready!
-	(*pgsdll_callback) (GSDLL_DEVICE, (char *)mdev, 0);
+	// it has to dispose the PICT handle when it is ready!
+	if (pgsdll_callback)
+		(*pgsdll_callback) (GSDLL_DEVICE, (char *)mdev, 0);
 	
 	return 0;
 }
