@@ -1,22 +1,28 @@
-/* Copyright (c) 2000 artofcode LLC.  All rights reserved
+/* Copyright (C) 1997-2002 artofcode LLC.  All rights reserved.                     
+
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; either version 2 of the License, or (at your
+  option) any later version.
+
+  This program is distributed in the hope that it will be useful, but
+  WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+  General Public License for more details.
+
+  You should have received a copy of the GNU General Public License along
+  with this program; if not, write to the Free Software Foundation, Inc.,
+  59 Temple Place, Suite 330, Boston, MA, 02111-1307.
 
-  This file is part of GNU Ghostscript.
-  
-  GNU Ghostscript is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY.  No author or distributor accepts responsibility
-  to anyone for the consequences of using it or for whether it serves any
-  particular purpose or works at all, unless he says so in writing.  Refer
-  to the GNU General Public License for full details.
-  
-  Everyone is granted permission to copy, modify and redistribute GNU
-  Ghostscript, but only under the conditions described in the GNU General
-  Public License.  A copy of this license is supposed to have been given
-  to you along with GNU Ghostscript so you can know your rights and
-  responsibilities.  It should be in a file named COPYING.  Among other
-  things, the copyright notice and this notice must be preserved on all
-  copies.
 */
 
+/* $Id: gdevmacpictop.h,v 1.1.4.2.2.1 2003/01/17 00:49:00 giles Exp $ */
+
+/* Helpers for working with Classic MacOS Quickdraw pictures */
+/* (obsoleted by the DISPLAY device) */
+
+#ifndef gdevmacpictop_INCLUDED
+#  define gdevmacpictop_INCLUDED
 
 #include <QDOffscreen.h>
 
@@ -681,5 +687,4 @@
 			PICT_RGBBkCol(ptr, rgb[0], rgb[1], rgb[2]);										\
 		}
 
-
-
+#endif /* gdevmacpictop_INCLUDED */
