@@ -1,20 +1,22 @@
-/* Copyright (C) 1994, 1995, 1997, 2001 Artifex Software, Inc.  All rights reserved.
+/* Copyright (C) 1994, 1995, 1997, 2001 artofcode LLC.  All rights reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; either version 2 of the License, or (at your
+  option) any later version.
+
+  This program is distributed in the hope that it will be useful, but
+  WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+  General Public License for more details.
+
+  You should have received a copy of the GNU General Public License along
+  with this program; if not, write to the Free Software Foundation, Inc.,
+  59 Temple Place, Suite 330, Boston, MA, 02111-1307.
+
 */
 
-/* $Id: gdevmacxf.c,v 1.1.4.2 2002/04/10 09:33:26 giles Exp $ */
+/* $Id: gdevmacxf.c,v 1.1.4.2.2.1 2003/01/17 00:49:00 giles Exp $ */
 /* External font (xfont) implementation for Classic/Carbon MacOS. */
 
 #include "gdevmac.h"
