@@ -1,20 +1,22 @@
-/* Copyright (C) 1992, 2000 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 1992, 2000 artofcode LLC.  All rights reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; either version 2 of the License, or (at your
+  option) any later version.
+
+  This program is distributed in the hope that it will be useful, but
+  WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+  General Public License for more details.
+
+  You should have received a copy of the GNU General Public License along
+  with this program; if not, write to the Free Software Foundation, Inc.,
+  59 Temple Place, Suite 330, Boston, MA, 02111-1307.
+
 */
 
-/*$Id: gdevpbm.c,v 1.3.6.1 2002/02/22 19:45:56 ray Exp $ */
+/*$Id: gdevpbm.c,v 1.3.6.2.2.1 2003/01/17 00:49:01 giles Exp $ */
 /* Portable Bit/Gray/PixMap drivers */
 #include "gdevprn.h"
 #include "gscdefs.h"
@@ -34,7 +36,7 @@
  */
 
 /*
- * There are 7 (families of) drivers here, plus one less related one:
+ * There are 8 (families of) drivers here, plus one less related one:
  *      pbm[raw] - outputs PBM (black and white).
  *      pgm[raw] - outputs PGM (gray-scale).
  *      pgnm[raw] - outputs PBM if the page contains only black and white,
@@ -45,6 +47,7 @@
  *        otherwise PPM.
  *      pkm[raw] - computes internally in CMYK, outputs PPM (RGB).
  *      pksm[raw] - computes internally in CMYK, outputs 4 PBM pages.
+ *      pam - outputs CMYK as PAM
  *      plan9bm - outputs Plan 9 bitmap format.
  */
 
@@ -59,6 +62,7 @@
  *      ppmraw, pnmraw: 4(3x1), 8(3x2), 16(3x5), 24(3x8).  [3-24]
  *      pkm, pkmraw: 4(4x1), 8(4x2), 16(4x4), 32(4x8).  [4-32]
  *	pksm, pksmraw: ibid.
+ *      pam: 32 (CMYK)
  */
 
 /* Structure for P*M devices, which extend the generic printer device. */
@@ -126,6 +130,12 @@ private dev_proc_print_page(pgm_print_page);
 private dev_proc_print_page(ppm_print_page);
 private dev_proc_print_page(pkm_print_page);
 private dev_proc_print_page(psm_print_page);
+private dev_proc_print_page(psm_print_page);
+private dev_proc_print_page(pam_print_page);
+
+private int pam_print_row(gx_device_printer * pdev, byte * data, int depth,
+	       FILE * pstream);
+private int pam_print_page(gx_device_printer * pdev, FILE * pstream);
 
 /* The device procedures */
 
@@ -152,6 +162,9 @@ private const gx_device_procs pnm_procs =
 private const gx_device_procs pkm_procs =
     pgpm_procs(ppm_open, ppm_get_params,
 	       NULL, cmyk_1bit_map_color_rgb, cmyk_1bit_map_cmyk_color);
+private const gx_device_procs pam_procs =
+    pgpm_procs(ppm_open, ppm_get_params,
+	       NULL, cmyk_8bit_map_color_rgb, cmyk_8bit_map_cmyk_color);
 
 /* The device descriptors themselves */
 const gx_device_pbm gs_pbm_device =
@@ -196,6 +209,9 @@ pbm_prn_device(pkm_procs, "pksm", '1', 0, 4, 4, 1, 1, 0,
 const gx_device_pbm gs_pksmraw_device =
 pbm_prn_device(pkm_procs, "pksmraw", '4', 1, 4, 4, 1, 1, 0,
 	       X_DPI, Y_DPI, psm_print_page);
+const gx_device_pbm gs_pam_device =
+pbm_prn_device(pam_procs, "pam", '7', 1, 4, 32, 255, 255, 0,
+	       X_DPI, Y_DPI, pam_print_page);
 
 /* Plan 9 bitmaps default to 100 dpi. */
 const gx_device_pbm gs_plan9bm_device =
@@ -220,7 +236,10 @@ ppm_set_dev_procs(gx_device * pdev)
 	set_dev_proc(pdev, begin_typed_image, pnm_begin_typed_image);
     }
     if (bdev->color_info.num_components == 4) {
-	if (bdev->color_info.depth == 4) {
+	if (bdev->magic == 7) {
+	    set_dev_proc(pdev, map_color_rgb, cmyk_8bit_map_color_rgb);
+	    set_dev_proc(pdev, map_cmyk_color, cmyk_8bit_map_cmyk_color);
+	} else if (bdev->color_info.depth == 4) {
 	    set_dev_proc(pdev, map_color_rgb, cmyk_1bit_map_color_rgb);
 	    set_dev_proc(pdev, map_cmyk_color, cmyk_1bit_map_cmyk_color);
 	} else {
@@ -524,6 +543,23 @@ pbm_print_page_loop(gx_device_printer * pdev, char magic, FILE * pstream,
     if (magic == '9')
 	fprintf(pstream, "%11d %11d %11d %11d %11d ",
 		0, 0, 0, pdev->width, pdev->height);
+    else if (magic == '7') {
+	int ncomps = pdev->color_info.num_components;
+	fprintf(pstream, "P%c\n", magic);
+	fprintf(pstream, "WIDTH %d\n", pdev->width);
+	fprintf(pstream, "HEIGHT %d\n", pdev->height);
+	fprintf(pstream, "DEPTH %d\n", ncomps);
+	fprintf(pstream, "MAXVAL %d\n", pdev->color_info.max_gray);
+	fprintf(pstream, "TUPLTYPE %s\n",
+	    (ncomps == 4) ? "CMYK" :
+	    ((ncomps == 3) ? "RGB" : "GRAYSCALE"));
+	if (bdev->comment[0])
+	    fprintf(pstream, "# %s\n", bdev->comment);
+	else
+	    fprintf(pstream, "# Image generated by %s (device=%s)\n",
+		    gs_product, pdev->dname);
+	fprintf(pstream, "ENDHDR\n");
+    }
     else {
 	fprintf(pstream, "P%c\n", magic);
 	if (bdev->comment[0])
@@ -536,6 +572,7 @@ pbm_print_page_loop(gx_device_printer * pdev, char magic, FILE * pstream,
     switch (magic) {
 	case '1':		/* pbm */
 	case '4':		/* pbmraw */
+	case '7':		/* pam */
 	case '9':		/* plan9bm */
 	    break;
 	default:
@@ -782,6 +819,24 @@ ppm_print_page(gx_device_printer * pdev, FILE * pstream)
 				pxm_pbm_print_row));
 }
 
+private int
+pam_print_row(gx_device_printer * pdev, byte * data, int depth,
+	       FILE * pstream)
+{
+    if (depth == 32)
+	fwrite(data, 1, pdev->width * (depth / 8), pstream);
+    return 0;
+}
+
+private int
+pam_print_page(gx_device_printer * pdev, FILE * pstream)
+{
+    gx_device_pbm * const bdev = (gx_device_pbm *)pdev;
+
+    return pbm_print_page_loop(pdev, bdev->magic, pstream,
+				pam_print_row);
+}
+
 /* Print a faux CMYK page. */
 /* Print a row where each pixel occupies 4 bits (depth == 4). */
 /* In this case, we also know pdev->color_info.max_color == 1. */
