@@ -1,20 +1,22 @@
-/* Copyright (C) 2000 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2000 artofcode LLC.  All rights reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; either version 2 of the License, or (at your
+  option) any later version.
+
+  This program is distributed in the hope that it will be useful, but
+  WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+  General Public License for more details.
+
+  You should have received a copy of the GNU General Public License along
+  with this program; if not, write to the Free Software Foundation, Inc.,
+  59 Temple Place, Suite 330, Boston, MA, 02111-1307.
+
 */
 
-/*$Id: gdevpdfe.c,v 1.10.2.1 2002/02/22 19:45:56 ray Exp $ */
+/*$Id: gdevpdfe.c,v 1.10.2.2.2.1 2003/01/17 00:49:01 giles Exp $ */
 /* Embedded font writing for pdfwrite driver. */
 #include "memory_.h"
 #include "string_.h"
@@ -118,7 +120,8 @@ pdf_embed_font_as_type1(gx_device_pdf *pdev, gs_font_type1 *font,
 private int
 pdf_embed_font_as_type2(gx_device_pdf *pdev, gs_font_type1 *font,
 			long FontFile_id, gs_glyph subset_glyphs[256],
-			uint subset_size, const gs_const_string *pfname)
+			uint subset_size, const gs_const_string *pfname,
+			gs_int_rect *FontBBox)
 {
     int code;
     pdf_data_writer_t writer;
@@ -130,7 +133,8 @@ pdf_embed_font_as_type2(gx_device_pdf *pdev, gs_font_type1 *font,
     if (code < 0)
 	return code;
     code = psf_write_type2_font(writer.binary.strm, font, options,
-				subset_glyphs, subset_size, pfname);
+				subset_glyphs, subset_size, pfname,
+				FontBBox);
     pdf_end_fontfile(pdev, &writer);
     return code;
 }
@@ -139,7 +143,8 @@ pdf_embed_font_as_type2(gx_device_pdf *pdev, gs_font_type1 *font,
 private int
 pdf_embed_font_type1(gx_device_pdf *pdev, gs_font_type1 *font,
 		     long FontFile_id, gs_glyph subset_glyphs[256],
-		     uint subset_size, const gs_const_string *pfname)
+		     uint subset_size, const gs_const_string *pfname,
+		     gs_int_rect *FontBBox)
 {
     switch (((const gs_font *)font)->FontType) {
     case ft_encrypted:
@@ -149,7 +154,7 @@ pdf_embed_font_type1(gx_device_pdf *pdev, gs_font_type1 *font,
 	/* For PDF 1.2 and later, write Type 1 fonts as Type1C. */
     case ft_encrypted2:
 	return pdf_embed_font_as_type2(pdev, font, FontFile_id,
-				       subset_glyphs, subset_size, pfname);
+			       subset_glyphs, subset_size, pfname, FontBBox);
     default:
 	return_error(gs_error_rangecheck);
     }
@@ -328,7 +333,8 @@ pdf_write_embedded_font(gx_device_pdf *pdev, pdf_font_descriptor_t *pfd)
 	    }
 	    code = pdf_embed_font_type1(pdev, (gs_font_type1 *)font,
 					FontFile_id, subset_list,
-					subset_size, &font_name);
+					subset_size, &font_name, 
+					&pfd->values.FontBBox);
 	    break;
 	case ft_TrueType:
 	    if (do_subset) {
