@@ -1,23 +1,22 @@
-/* Copyright (C) 2000 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 2000 artofcode LLC.  All rights reserved.
   
-  This file is part of GNU Ghostscript.
-  
-  GNU Ghostscript is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY.  No author or distributor accepts responsibility
-  to anyone for the consequences of using it or for whether it serves any
-  particular purpose or works at all, unless he says so in writing.  Refer
-  to the GNU General Public License for full details.
-  
-  Everyone is granted permission to copy, modify and redistribute GNU
-  Ghostscript, but only under the conditions described in the GNU General
-  Public License.  A copy of this license is supposed to have been given
-  to you along with GNU Ghostscript so you can know your rights and
-  responsibilities.  It should be in a file named COPYING.  Among other
-  things, the copyright notice and this notice must be preserved on all
-  copies.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; either version 2 of the License, or (at your
+  option) any later version.
+
+  This program is distributed in the hope that it will be useful, but
+  WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+  General Public License for more details.
+
+  You should have received a copy of the GNU General Public License along
+  with this program; if not, write to the Free Software Foundation, Inc.,
+  59 Temple Place, Suite 330, Boston, MA, 02111-1307.
+
 */
 
-/*$RCSfile: gdevpdfv.c,v $ $Revision: 1.3.2.3 $ */
+/*$Id: gdevpdfv.c,v 1.10.2.1.2.1 2003/01/17 00:49:01 giles Exp $ */
 /* Color value writing for pdfwrite driver */
 #include "math_.h"
 #include "string_.h"
@@ -71,6 +70,20 @@ tile_size(const gx_strip_bitmap *tile, int depth)
 {
     return (tile->rep_width * depth + 7) / 8 * tile->rep_height;
 }
+private bool
+tile_size_ok(const gx_device_pdf *pdev, const gx_color_tile *p_tile,
+	     const gx_color_tile *m_tile)
+{
+    /*
+     * Acrobat Reader can't handle image Patterns with more than
+     * 64K of data.  :-(
+     */
+    uint p_size =
+	(p_tile == 0 ? 0 : tile_size(&p_tile->tbits, pdev->color_info.depth));
+    uint m_size =
+	(m_tile == 0 ? 0 : tile_size(&m_tile->tmask, 1));
+    return (max(p_size, m_size) <= 65500);
+}
 private int
 pdf_pattern(gx_device_pdf *pdev, const gx_drawing_color *pdc,
 	    const gx_color_tile *p_tile, const gx_color_tile *m_tile,
@@ -84,21 +97,13 @@ pdf_pattern(gx_device_pdf *pdev, const gx_drawing_color *pdc,
     cos_dict_t *pcd_Resources = cos_dict_alloc(pdev, "pdf_pattern(Resources)");
     const gx_color_tile *tile = (p_tile ? p_tile : m_tile);
     const gx_strip_bitmap *btile = (p_tile ? &p_tile->tbits : &m_tile->tmask);
-    uint p_size =
-	(p_tile == 0 ? 0 : tile_size(&p_tile->tbits, pdev->color_info.depth));
-    uint m_size =
-	(m_tile == 0 ? 0 : tile_size(&m_tile->tmask, 1));
     bool mask = p_tile == 0;
     gs_point step;
     gs_matrix smat;
 
     if (code < 0)
 	return code;
-    /*
-     * Acrobat Reader can't handle image Patterns with more than
-     * 64K of data.  :-(
-     */
-    if (max(p_size, m_size) > 65500)
+    if (!tile_size_ok(pdev, p_tile, m_tile))
 	return_error(gs_error_limitcheck);
     /*
      * We currently can't handle Patterns whose X/Y step isn't parallel
@@ -171,7 +176,7 @@ pdf_set_pattern_image(gs_data_image_t *pic, const gx_strip_bitmap *tile)
     pic->ImageMatrix.yy = pic->Height = tile->rep_height;
 }
 
-/* Write the mask for a Pattern. */
+/* Write the mask for a Pattern (colored or uncolored). */
 private int
 pdf_put_pattern_mask(gx_device_pdf *pdev, const gx_color_tile *m_tile,
 		     cos_stream_t **ppcs_mask)
@@ -204,6 +209,44 @@ pdf_put_pattern_mask(gx_device_pdf *pdev, const gx_color_tile *m_tile,
     return 0;
 }
 
+/* Write an uncolored Pattern color. */
+/* (Single-use procedure for readability.) */
+private int
+pdf_put_uncolored_pattern(gx_device_pdf *pdev, const gx_drawing_color *pdc,
+			  const psdf_set_color_commands_t *ppscc,
+			  pdf_resource_t **ppres)
+{
+    const gx_color_tile *m_tile = pdc->mask.m_tile;
+    gx_drawing_color dc_pure;
+
+    color_set_pure(&dc_pure, gx_dc_pure_color(pdc));
+    if (m_tile == 0) {
+	/*
+	 * If m_tile == 0, this uncolored Pattern is all 1's,
+	 * equivalent to a pure color.
+	 */
+	*ppres = 0;
+	return psdf_set_color((gx_device_vector *)pdev, &dc_pure, ppscc);
+    } else {
+	cos_value_t v;
+	stream *s = pdev->strm;
+	int code;
+	cos_stream_t *pcs_image;
+	static const psdf_set_color_commands_t no_scc = {0, 0, 0};
+
+	if (!tile_size_ok(pdev, NULL, m_tile))
+	    return_error(gs_error_limitcheck);
+	if ((code = pdf_cs_Pattern_uncolored(pdev, &v)) < 0 ||
+	    (code = pdf_put_pattern_mask(pdev, m_tile, &pcs_image)) < 0 ||
+	    (code = pdf_pattern(pdev, pdc, NULL, m_tile, pcs_image, ppres)) < 0
+	    )
+	    return code;
+	cos_value_write(&v, pdev);
+	pprints1(s, " %s ", ppscc->setcolorspace);
+	return psdf_set_color((gx_device_vector *)pdev, &dc_pure, &no_scc);
+    }
+}
+
 /* Write a colored Pattern color. */
 /* (Single-use procedure for readability.) */
 private int
@@ -222,8 +265,67 @@ pdf_put_colored_pattern(gx_device_pdf *pdev, const gx_drawing_color *pdc,
     cos_stream_t *pcs_mask = 0;
     cos_value_t v;
     long pos;
-    int code = pdf_cs_Pattern_colored(pdev, &v);
+    int code;
 
+    /*
+     * NOTE: We assume here that the color space of the cached Pattern
+     * is the same as the native color space of the device.  This will
+     * have to change in the future!
+     */
+    /*
+     * Check whether this colored pattern is actually a masked pure color,
+     * by testing whether all the colored pixels have the same color.
+     */
+    if (m_tile) {
+	if (p_tile && !(p_tile->depth & 7) && p_tile->depth <= sizeof(gx_color_index) * 8) {
+	    int depth_bytes = p_tile->depth >> 3;
+	    int width = p_tile->tbits.rep_width;
+	    int skip = p_tile->tbits.raster -
+		p_tile->tbits.rep_width * depth_bytes;
+	    const byte *bp;
+	    const byte *mp;
+	    int i, j, k;
+	    gx_color_index color = 0; /* init is arbitrary if not empty */
+	    bool first = true;
+
+	    for (i = 0, bp = p_tile->tbits.data, mp = m_tile->tbits.data;
+		 i < p_tile->tbits.rep_height;
+		 ++i, bp += skip, mp += m_tile->tbits.raster) {
+
+		for (j = 0; j < width; ++j) {
+		    if (mp[j >> 3] & (0x80 >> (j & 7))) {
+			gx_color_index ci = 0;
+		    
+			for (k = 0; k < depth_bytes; ++k)
+			    ci = (ci << 8) + *bp++;
+			if (first)
+			    color = ci, first = false;
+			else if (ci != color)
+			    goto not_pure;
+		    } else
+			bp += depth_bytes;
+		}
+	    }
+	    {
+		/* Set the color, then handle as an uncolored pattern. */
+		gx_drawing_color dcolor;
+
+		dcolor = *pdc;
+		dcolor.colors.pure = color;
+		return pdf_put_uncolored_pattern(pdev, &dcolor, ppscc, ppres);
+	    }
+	not_pure:
+	    DO_NOTHING;		/* required by MSVC */
+	}
+	if (pdev->CompatibilityLevel < 1.3) {
+	    /* Masked images are only supported starting in PDF 1.3. */
+	    return_error(gs_error_rangecheck);
+	}
+    }
+    /* Acrobat Reader has a size limit for image Patterns. */
+    if (!tile_size_ok(pdev, p_tile, m_tile))
+	return_error(gs_error_limitcheck);
+    code = pdf_cs_Pattern_colored(pdev, &v);
     if (code < 0)
 	return code;
     pdf_cspace_init_Device(&cs_Device, pdev->color_info.num_components);
@@ -268,33 +370,6 @@ pdf_put_colored_pattern(gx_device_pdf *pdev, const gx_drawing_color *pdc,
     return 0;
 }
 
-/* Write an uncolored Pattern color. */
-/* (Single-use procedure for readability.) */
-private int
-pdf_put_uncolored_pattern(gx_device_pdf *pdev, const gx_drawing_color *pdc,
-			  const psdf_set_color_commands_t *ppscc,
-			  pdf_resource_t **ppres)
-{
-    const gx_color_tile *m_tile = pdc->mask.m_tile;
-    cos_value_t v;
-    stream *s = pdev->strm;
-    int code = pdf_cs_Pattern_uncolored(pdev, &v);
-    cos_stream_t *pcs_image;
-    gx_drawing_color dc_pure;
-    static const psdf_set_color_commands_t no_scc = {0, 0, 0};
-
-    if (code < 0 ||
-	(code = pdf_put_pattern_mask(pdev, m_tile, &pcs_image)) < 0 ||
-	(code = pdf_pattern(pdev, pdc, NULL, m_tile, pcs_image, ppres)) < 0
-	)
-	return code;
-    cos_value_write(&v, pdev);
-    pprints1(s, " %s ", ppscc->setcolorspace);
-    color_set_pure(&dc_pure, gx_dc_pure_color(pdc));
-    psdf_set_color((gx_device_vector *)pdev, &dc_pure, &no_scc);
-    return 0;
-}
-
 /* ---------------- PatternType 2 colors ---------------- */
 
 /* Write parameters common to all Shadings. */
@@ -683,7 +758,7 @@ pdf_put_pattern2(gx_device_pdf *pdev, const gx_drawing_color *pdc,
 
 /* ---------------- Public procedure ---------------- */
 
-/* Write a color value.  rgs is "rg" for fill, "RG" for stroke. */
+/* Write a color value. */
 int
 pdf_put_drawing_color(gx_device_pdf *pdev, const gx_drawing_color *pdc,
 		      const psdf_set_color_commands_t *ppscc)
@@ -706,8 +781,14 @@ pdf_put_drawing_color(gx_device_pdf *pdev, const gx_drawing_color *pdc,
 	    return_error(gs_error_rangecheck);
 	if (code < 0)
 	    return code;
-	cos_value_write(cos_resource_value(&v, pres->object), pdev);
-	pprints1(pdev->strm, " %s\n", ppscc->setcolorn);
+	/*
+	 * Uncolored patterns equivalent to a solid color don't need an
+	 * associated resource, and set pres = 0.
+	 */
+	if (pres) {
+	    cos_value_write(cos_resource_value(&v, pres->object), pdev);
+	    pprints1(pdev->strm, " %s\n", ppscc->setcolorn);
+	}
 	return 0;
     }
 }
