@@ -1,20 +1,22 @@
-/* Copyright (C) 1989, 1995, 1996, 1997, 1998, 1999, 2000 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 1989, 1995, 1996, 1997, 1998, 1999, 2000 artofcode LLC.  All rights reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; either version 2 of the License, or (at your
+  option) any later version.
+
+  This program is distributed in the hope that it will be useful, but
+  WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+  General Public License for more details.
+
+  You should have received a copy of the GNU General Public License along
+  with this program; if not, write to the Free Software Foundation, Inc.,
+  59 Temple Place, Suite 330, Boston, MA, 02111-1307.
+
 */
 
-/*$Id: gdevprn.h,v 1.6.6.1 2002/02/22 19:45:56 ray Exp $ */
+/*$Id: gdevprn.h,v 1.6.6.2.2.2 2003/03/11 11:59:10 ghostgum Exp $ */
 /* Common header file for memory-buffered printers */
 
 #ifndef gdevprn_INCLUDED
@@ -331,6 +333,7 @@ prn_dev_proc_buffer_page(gx_default_buffer_page); /* returns an error */
 	NULL,	/* strip_copy_rop, */\
 	NULL,	/* get_clipping_box */\
 	NULL,	/* begin_typed_image */\
+	NULL,	/* get_bits_rectangle */\
 	NULL,	/* map_color_rgb_alpha */\
 	NULL,	/* create_compositor */\
 	NULL,	/* get_hardware_params */\
@@ -393,13 +396,13 @@ extern const gx_device_procs prn_std_procs;
 /* This accounts for the short parameter names here and below. */
 #define prn_device_margins_body(dtype, procs, dname, w10, h10, xdpi, ydpi, lo, to, lm, bm, rm, tm, ncomp, depth, mg, mc, dg, dc, print_page)\
 	std_device_full_body_type(dtype, &procs, dname, &st_device_printer,\
-	  (int)((long)(w10) * (xdpi) / 10),\
-	  (int)((long)(h10) * (ydpi) / 10),\
+	  (int)((float)(w10) * (xdpi) / 10 + 0.5),\
+	  (int)((float)(h10) * (ydpi) / 10 + 0.5),\
 	  xdpi, ydpi,\
 	  ncomp, depth, mg, mc, dg, dc,\
-	  -(lo) * (xdpi), -(to) * (ydpi),\
-	  (lm) * 72.0, (bm) * 72.0,\
-	  (rm) * 72.0, (tm) * 72.0\
+	  (float)(-(lo) * (xdpi)), (float)(-(to) * (ydpi)),\
+	  (float)((lm) * 72.0), (float)((bm) * 72.0),\
+	  (float)((rm) * 72.0), (float)((tm) * 72.0)\
 	),\
 	prn_device_body_rest_(print_page)
 
@@ -409,12 +412,12 @@ extern const gx_device_procs prn_std_procs;
 
 #define prn_device_std_margins_body(dtype, procs, dname, w10, h10, xdpi, ydpi, lo, to, lm, bm, rm, tm, color_bits, print_page)\
 	std_device_std_color_full_body_type(dtype, &procs, dname, &st_device_printer,\
-	  (int)((long)(w10) * (xdpi) / 10),\
-	  (int)((long)(h10) * (ydpi) / 10),\
+	  (int)((float)(w10) * (xdpi) / 10 + 0.5),\
+	  (int)((float)(h10) * (ydpi) / 10 + 0.5),\
 	  xdpi, ydpi, color_bits,\
-	  -(lo) * (xdpi), -(to) * (ydpi),\
-	  (lm) * 72.0, (bm) * 72.0,\
-	  (rm) * 72.0, (tm) * 72.0\
+	  (float)(-(lo) * (xdpi)), (float)(-(to) * (ydpi)),\
+	  (float)((lm) * 72.0), (float)((bm) * 72.0),\
+	  (float)((rm) * 72.0), (float)((tm) * 72.0)\
 	),\
 	prn_device_body_rest_(print_page)
 
@@ -424,12 +427,12 @@ extern const gx_device_procs prn_std_procs;
 
 #define prn_device_std_margins_body_copies(dtype, procs, dname, w10, h10, xdpi, ydpi, lo, to, lm, bm, rm, tm, color_bits, print_page_copies)\
 	std_device_std_color_full_body_type(dtype, &procs, dname, &st_device_printer,\
-	  (int)((long)(w10) * (xdpi) / 10),\
-	  (int)((long)(h10) * (ydpi) / 10),\
+	  (int)((float)(w10) * (xdpi) / 10 + 0.5),\
+	  (int)((float)(h10) * (ydpi) / 10 + 0.5),\
 	  xdpi, ydpi, color_bits,\
-	  -(lo) * (xdpi), -(to) * (ydpi),\
-	  (lm) * 72.0, (bm) * 72.0,\
-	  (rm) * 72.0, (tm) * 72.0\
+	  (float)(-(lo) * (xdpi)), (float)(-(to) * (ydpi)),\
+	  (float)((lm) * 72.0), (float)((bm) * 72.0),\
+	  (float)((rm) * 72.0), (float)((tm) * 72.0)\
 	),\
 	prn_device_body_copies_rest_(print_page_copies)
 
