@@ -1,23 +1,22 @@
-/* Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999 artofcode LLC.  All rights reserved.
   
-  This file is part of GNU Ghostscript.
-  
-  GNU Ghostscript is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY.  No author or distributor accepts responsibility
-  to anyone for the consequences of using it or for whether it serves any
-  particular purpose or works at all, unless he says so in writing.  Refer
-  to the GNU General Public License for full details.
-  
-  Everyone is granted permission to copy, modify and redistribute GNU
-  Ghostscript, but only under the conditions described in the GNU General
-  Public License.  A copy of this license is supposed to have been given
-  to you along with GNU Ghostscript so you can know your rights and
-  responsibilities.  It should be in a file named COPYING.  Among other
-  things, the copyright notice and this notice must be preserved on all
-  copies.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; either version 2 of the License, or (at your
+  option) any later version.
+
+  This program is distributed in the hope that it will be useful, but
+  WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+  General Public License for more details.
+
+  You should have received a copy of the GNU General Public License along
+  with this program; if not, write to the Free Software Foundation, Inc.,
+  59 Temple Place, Suite 330, Boston, MA, 02111-1307.
+
 */
 
-/*$RCSfile: gdevxalt.c,v $ $Revision: 1.2.2.1 $ */
+/*$Id: gdevxalt.c,v 1.2.6.2.2.1 2003/01/17 00:49:02 giles Exp $ */
 /* Alternative X Windows drivers for help in driver debugging */
 #include "gx.h"			/* for gx_bitmap; includes std.h */
 #include "math_.h"
@@ -709,6 +708,7 @@ x_gray_alt_map_color(gx_device * dev, gx_color_index color,
     return -1;
 }
 
+#ifdef USE_BROKEN_X11ALPHA
 /* ---------------- Alpha procedures ---------------- */
 
 /* Device procedures */
@@ -761,7 +761,7 @@ x_alpha_alt_map_color(gx_device * dev, gx_color_index color,
 {
     return color & 0xffffff;	/* just remove alpha */
 }
-
+#endif
 /* Device procedures */
 
 /* We encode a complemented alpha value in the top 8 bits of the */
@@ -783,6 +783,7 @@ x_alpha_map_rgb_alpha_color(gx_device * dev,
 	    ((gx_color_index) (abyte ^ 0xff) << 24) + color);
 }
 
+#ifdef USE_BROKEN_X11ALPHA
 private int
 x_alpha_copy_alpha(gx_device * dev, const unsigned char *base, int sourcex,
 		   int raster, gx_bitmap_id id, int x, int y, int w, int h,
@@ -857,6 +858,7 @@ x_alpha_copy_alpha(gx_device * dev, const unsigned char *base, int sourcex,
     }
     return 0;
 }
+#endif
 
 /* ---------------- Permuted RGB16/32 procedures ---------------- */
 
