@@ -1,20 +1,22 @@
-/* Copyright (C) 1991, 2000 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 1991, 2000 artofcode LLC.  All rights reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; either version 2 of the License, or (at your
+  option) any later version.
+
+  This program is distributed in the hope that it will be useful, but
+  WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+  General Public License for more details.
+
+  You should have received a copy of the GNU General Public License along
+  with this program; if not, write to the Free Software Foundation, Inc.,
+  59 Temple Place, Suite 330, Boston, MA, 02111-1307.
+
 */
 
-/*$Id: gp.h,v 1.8.2.2 2002/02/22 19:45:57 ray Exp $ */
+/*$Id: gp.h,v 1.8.2.3.2.1 2003/01/17 00:49:02 giles Exp $ */
 /* Interface to platform-specific routines */
 /* Requires gsmemory.h */
 
@@ -179,9 +181,19 @@ FILE *gp_fopen(P2(const char *fname, const char *mode));
 /* if 2nd param true, text mode if 2nd param false */
 int gp_setmode_binary(P2(FILE * pfile, bool mode));
 
-/* Answer whether a file name contains a directory/device specification, */
-/* i.e. is absolute (not directory- or device-relative). */
-bool gp_file_name_is_absolute(P2(const char *fname, uint len));
+/* Answer whether a path string is not "bare" (returns true) i.e.,	*/
+/* contains an absolute reference, an initial 'current directory' ref	*/
+/* or some form of relative reference to the parent directory. The	*/
+/* "non_bare" pathstrings are those for	which it is not valid to prefix	*/
+/* a path (and expect the result to still be meaningful/valid).		*/
+/*									*/
+/* Some examples of non-bare pathstrings platform variants are:		*/
+/*	unix:	starts with '/' or './', or contains '../'		*/
+/*	mac:	starts with ':', or contains '::'			*/
+/*	VMS:	contains (starts with) directory '[  ]' spec.		*/
+/*	Win:	contains initial drive letter, initial '.', '/' or '\'	*/
+/*		or contains '../' or '..\'				*/
+bool gp_pathstring_not_bare(P2(const char *fname, uint len));
 
 /* Answer whether a file name contains a parent directory reference, */
 /* e.g., "../somefile". Currently used for security purposes. */
