@@ -1,20 +1,22 @@
-/* Copyright (C) 1992, 2000 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 1992, 2000 artofcode LLC.  All rights reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; either version 2 of the License, or (at your
+  option) any later version.
+
+  This program is distributed in the hope that it will be useful, but
+  WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+  General Public License for more details.
+
+  You should have received a copy of the GNU General Public License along
+  with this program; if not, write to the Free Software Foundation, Inc.,
+  59 Temple Place, Suite 330, Boston, MA, 02111-1307.
+
 */
 
-/*$Id: gp_mswin.c,v 1.7.2.1 2002/02/22 19:45:57 ray Exp $ */
+/*$Id: gp_mswin.c,v 1.7.2.3.2.1 2003/01/17 00:49:02 giles Exp $ */
 /*
  * Microsoft Windows platform support for Ghostscript.
  *
@@ -33,6 +35,7 @@
 #include "stdio_.h"
 #include "string_.h"
 #include "memory_.h"
+#include "pipe_.h"
 #include <stdlib.h>
 #include <stdarg.h>
 #include "ctype_.h"
@@ -111,7 +114,9 @@ gp_open_printer(char fname[gp_file_name_sizeof], int binary_mode)
 	pfile = gp_open_scratch_file(gp_scratch_file_name_prefix,
 				     win_prntmp, "wb");
 	return pfile;
-    } else
+    } else if (fname[0] == '|') 	/* pipe */
+	return popen(fname + 1, (binary_mode ? "wb" : "w"));
+    else
 	return fopen(fname, (binary_mode ? "wb" : "w"));
 }
 
@@ -551,6 +556,116 @@ gp_printfile_gs16spl(const char *filename, const char *port)
 }
 
 
+/******************************************************************/
+/* MS Windows has popen and pclose in stdio.h, but under different names.
+ * Unfortunately MSVC5 and 6 have a broken implementation of _popen, 
+ * so we use own.  Our implementation only supports mode "wb".
+ */
+FILE *mswin_popen(const char *cmd, const char *mode)
+{
+    SECURITY_ATTRIBUTES saAttr;
+    STARTUPINFO siStartInfo;
+    PROCESS_INFORMATION piProcInfo;
+    HANDLE hPipeTemp = INVALID_HANDLE_VALUE;
+    HANDLE hChildStdinRd = INVALID_HANDLE_VALUE;
+    HANDLE hChildStdinWr = INVALID_HANDLE_VALUE;
+    HANDLE hChildStdoutWr = INVALID_HANDLE_VALUE;
+    HANDLE hChildStderrWr = INVALID_HANDLE_VALUE;
+    HANDLE hProcess = GetCurrentProcess();
+    int handle = 0;
+    char *command = NULL;
+    FILE *pipe = NULL;
+
+    if (strcmp(mode, "wb") != 0)
+	return NULL;
+
+    /* Set the bInheritHandle flag so pipe handles are inherited. */
+    saAttr.nLength = sizeof(SECURITY_ATTRIBUTES);
+    saAttr.bInheritHandle = TRUE;
+    saAttr.lpSecurityDescriptor = NULL;
+
+    /* Create anonymous inheritable pipes for STDIN for child. 
+     * First create a noninheritable duplicate handle of our end of 
+     * the pipe, then close the inheritable handle.
+     */
+    if (handle == 0)
+        if (!CreatePipe(&hChildStdinRd, &hPipeTemp, &saAttr, 0))
+	    handle = -1;
+    if (handle == 0) {
+	if (!DuplicateHandle(hProcess, hPipeTemp, 
+	    hProcess, &hChildStdinWr, 0, FALSE /* not inherited */,
+	    DUPLICATE_SAME_ACCESS))
+	    handle = -1;
+        CloseHandle(hPipeTemp);
+    }
+    /* Create inheritable duplicate handles for our stdout/err */
+    if (handle == 0)
+	if (!DuplicateHandle(hProcess, GetStdHandle(STD_OUTPUT_HANDLE),
+	    hProcess, &hChildStdoutWr, 0, TRUE /* inherited */,
+	    DUPLICATE_SAME_ACCESS))
+	    handle = -1;
+    if (handle == 0)
+        if (!DuplicateHandle(hProcess, GetStdHandle(STD_ERROR_HANDLE),
+            hProcess, &hChildStderrWr, 0, TRUE /* inherited */, 
+	    DUPLICATE_SAME_ACCESS))
+	    handle = -1;
+
+    /* Set up members of STARTUPINFO structure. */
+    memset(&siStartInfo, 0, sizeof(STARTUPINFO));
+    siStartInfo.cb = sizeof(STARTUPINFO);
+    siStartInfo.dwFlags = STARTF_USESTDHANDLES;
+    siStartInfo.hStdInput = hChildStdinRd;
+    siStartInfo.hStdOutput = hChildStdoutWr;
+    siStartInfo.hStdError = hChildStderrWr;
+
+    if (handle == 0) {
+	command = (char *)malloc(strlen(cmd)+1);
+	if (command)
+	    strcpy(command, cmd);
+	else
+	    handle = -1;
+    }
+
+    if (handle == 0)
+	if (!CreateProcess(NULL,
+	    command,  	   /* command line                       */
+	    NULL,          /* process security attributes        */
+	    NULL,          /* primary thread security attributes */
+	    TRUE,          /* handles are inherited              */
+	    0,             /* creation flags                     */
+	    NULL,          /* environment                        */
+	    NULL,          /* use parent's current directory     */
+	    &siStartInfo,  /* STARTUPINFO pointer                */
+	    &piProcInfo))  /* receives PROCESS_INFORMATION  */ 
+	{
+	    handle = -1;
+	}
+	else {
+	    CloseHandle(piProcInfo.hProcess);
+	    CloseHandle(piProcInfo.hThread);
+	    handle = _open_osfhandle((long)hChildStdinWr, 0);
+	}
+
+    if (hChildStdinRd != INVALID_HANDLE_VALUE)
+	CloseHandle(hChildStdinRd);	/* close our copy */
+    if (hChildStdoutWr != INVALID_HANDLE_VALUE)
+	CloseHandle(hChildStdoutWr);	/* close our copy */
+    if (hChildStderrWr != INVALID_HANDLE_VALUE)
+	CloseHandle(hChildStderrWr);	/* close our copy */
+    if (command)
+	free(command);
+
+    if (handle < 0) {
+	if (hChildStdinWr != INVALID_HANDLE_VALUE)
+	    CloseHandle(hChildStdinWr);
+    }
+    else {
+	pipe = _fdopen(handle, "wb");
+	if (pipe == NULL)
+	    _close(handle);
+    }
+    return pipe;
+}
 
 
 /* ------ File naming and accessing ------ */
@@ -563,7 +678,7 @@ gp_open_scratch_file(const char *prefix, char *fname, const char *mode)
     int prefix_length = strlen(prefix);
     int len = gp_file_name_sizeof - prefix_length - 7;
 
-    if (gp_file_name_is_absolute(prefix, prefix_length) ||
+    if (gp_pathstring_not_bare(prefix, prefix_length) ||
 	gp_gettmpdir(fname, &len) != 0
 	)
 	*fname = 0;
