@@ -1,20 +1,22 @@
-/* Copyright (C) 1995, 2000 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 1995, 2000 artofcode LLC.  All rights reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; either version 2 of the License, or (at your
+  option) any later version.
+
+  This program is distributed in the hope that it will be useful, but
+  WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+  General Public License for more details.
+
+  You should have received a copy of the GNU General Public License along
+  with this program; if not, write to the Free Software Foundation, Inc.,
+  59 Temple Place, Suite 330, Boston, MA, 02111-1307.
+
 */
 
-/*$Id: gsalloc.c,v 1.11.2.1 2002/02/22 19:45:57 ray Exp $ */
+/*$Id: gsalloc.c,v 1.11.2.2.2.1 2003/01/17 00:49:02 giles Exp $ */
 /* Standard memory allocator */
 #include "gx.h"
 #include "memory_.h"
@@ -91,7 +93,7 @@ private RELOC_PTRS_WITH(ref_memory_reloc_ptrs, gs_ref_memory_t *mptr)
 RELOC_PTRS_END
 
 /*
- * Define flags for the alloc_obj, which implements all but the fastest
+ * Define the flags for alloc_obj, which implements all but the fastest
  * case of allocation.
  */
 typedef enum {
@@ -195,7 +197,10 @@ ialloc_alloc_state(gs_raw_memory_t * parent, uint chunk_size)
     iimem->gc_status.vm_threshold = chunk_size * 3L;
     iimem->gc_status.max_vm = max_long;
     iimem->gc_status.psignal = NULL;
+    iimem->gc_status.signal_value = 0;
     iimem->gc_status.enabled = false;
+    iimem->gc_status.requested = 0;
+    iimem->gc_allocated = 0;
     iimem->previous_status.allocated = 0;
     iimem->previous_status.used = 0;
     ialloc_reset(iimem);
