@@ -1,23 +1,22 @@
-/* Copyright (C) 1989, 1995, 1996, 1997, 1998, 1999 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 1989, 1995, 1996, 1997, 1998, 1999 artofcode LLC.  All rights reserved.
   
-  This file is part of GNU Ghostscript.
-  
-  GNU Ghostscript is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY.  No author or distributor accepts responsibility
-  to anyone for the consequences of using it or for whether it serves any
-  particular purpose or works at all, unless he says so in writing.  Refer
-  to the GNU General Public License for full details.
-  
-  Everyone is granted permission to copy, modify and redistribute GNU
-  Ghostscript, but only under the conditions described in the GNU General
-  Public License.  A copy of this license is supposed to have been given
-  to you along with GNU Ghostscript so you can know your rights and
-  responsibilities.  It should be in a file named COPYING.  Among other
-  things, the copyright notice and this notice must be preserved on all
-  copies.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; either version 2 of the License, or (at your
+  option) any later version.
+
+  This program is distributed in the hope that it will be useful, but
+  WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+  General Public License for more details.
+
+  You should have received a copy of the GNU General Public License along
+  with this program; if not, write to the Free Software Foundation, Inc.,
+  59 Temple Place, Suite 330, Boston, MA, 02111-1307.
+
 */
 
-/*$RCSfile: gsfont.c,v $ $Revision: 1.2.2.2 $ */
+/*$Id: gsfont.c,v 1.4.4.1.2.1 2003/01/17 00:49:02 giles Exp $ */
 /* Font operators for Ghostscript library */
 #include "gx.h"
 #include "memory_.h"
@@ -690,21 +689,16 @@ gs_default_font_info(gs_font *font, const gs_point *pscale, int members,
 		 index != 0;
 	     ) {
 	    gs_glyph_info_t glyph_info;
-	    gs_const_string gnstr;
 
 	    code = font->procs.glyph_info(font, glyph, pmat,
 					  (GLYPH_INFO_WIDTH0 << wmode),
 					  &glyph_info);
 	    if (code < 0)
 		return code;
-	    if (notdef == gs_no_glyph) {
-		gnstr.data = (const byte *)
-		    bfont->procs.callbacks.glyph_name(glyph, &gnstr.size);
-		if (gnstr.size == 7 && !memcmp(gnstr.data, ".notdef", 7)) {
-		    notdef = glyph;
-		    info->MissingWidth = glyph_info.width[wmode].x;
-		    info->members |= FONT_INFO_MISSING_WIDTH;
-		}
+	    if (notdef == gs_no_glyph && gs_font_glyph_is_notdef(bfont, glyph)) {
+		notdef = glyph;
+		info->MissingWidth = glyph_info.width[wmode].x;
+		info->members |= FONT_INFO_MISSING_WIDTH;
 	    }
 	    if (glyph_info.width[wmode].y != 0)
 		fixed_width = min_int;
@@ -734,17 +728,8 @@ gs_default_font_info(gs_font *font, const gs_point *pscale, int members,
 	     * If this is a CIDFont or TrueType font that uses integers as
 	     * glyph names, check for glyph 0; otherwise, check for .notdef.
 	     */
-	    if (glyph >= gs_min_cid_glyph) {
-		if (glyph != gs_min_cid_glyph)
-		    continue;
-	    } else {
-		gs_const_string gnstr;
-
-		gnstr.data = (const byte *)
-		    bfont->procs.callbacks.glyph_name(glyph, &gnstr.size);
-		if (gnstr.size != 7 || memcmp(gnstr.data, ".notdef", 7))
-		    continue;
-	    }
+	    if (!gs_font_glyph_is_notdef(bfont, glyph))
+		continue;
 	    {
 		gs_glyph_info_t glyph_info;
 		int code = font->procs.glyph_info(font, glyph, pmat,
@@ -798,6 +783,24 @@ gs_base_same_font(const gs_font *font, const gs_font *ofont, int mask)
 
 /* ------ Glyph-level procedures ------ */
 
+/*
+ * Test whether a glyph is the notdef glyph for a base font.
+ * The test is somewhat adhoc: perhaps this should be a virtual procedure.
+ */
+bool
+gs_font_glyph_is_notdef(gs_font_base *bfont, gs_glyph glyph)
+{
+    gs_const_string gnstr;
+
+    if (glyph == gs_no_glyph)
+	return false;
+    if (glyph >= gs_min_cid_glyph)
+	return (glyph == gs_min_cid_glyph);
+    gnstr.data = (const byte *)
+	bfont->procs.callbacks.glyph_name(glyph, &gnstr.size);
+    return (gnstr.size == 7 && !memcmp(gnstr.data, ".notdef", 7));
+}
+
 /* Dummy character encoding procedure */
 gs_glyph
 gs_no_encode_char(gs_font *pfont, gs_char chr, gs_glyph_space_t glyph_space)
