@@ -1,23 +1,22 @@
-/* Copyright (C) 1993, 2002 artofcode LLC.  All rights reserved.
+/* Copyright (C) 1993, 2000 artofcode LLC.  All rights reserved.
   
-  This file is part of GNU Ghostscript.
-  
-  GNU Ghostscript is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY.  No author or distributor accepts responsibility
-  to anyone for the consequences of using it or for whether it serves any
-  particular purpose or works at all, unless he says so in writing.  Refer
-  to the GNU General Public License for full details.
-  
-  Everyone is granted permission to copy, modify and redistribute GNU
-  Ghostscript, but only under the conditions described in the GNU General
-  Public License.  A copy of this license is supposed to have been given
-  to you along with GNU Ghostscript so you can know your rights and
-  responsibilities.  It should be in a file named COPYING.  Among other
-  things, the copyright notice and this notice must be preserved on all
-  copies.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; either version 2 of the License, or (at your
+  option) any later version.
+
+  This program is distributed in the hope that it will be useful, but
+  WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+  General Public License for more details.
+
+  You should have received a copy of the GNU General Public License along
+  with this program; if not, write to the Free Software Foundation, Inc.,
+  59 Temple Place, Suite 330, Boston, MA, 02111-1307.
+
 */
 
-/* $Id: gsparam.h,v 1.7.2.2 2002/01/23 04:15:50 giles Exp $ */
+/*$Id: gsparam.h,v 1.8.4.1.2.1 2003/01/17 00:49:03 giles Exp $ */
 /* Client interface to parameter dictionaries */
 
 #ifndef gsparam_INCLUDED
@@ -85,7 +84,7 @@ typedef _param_array_struct(gs_param_string_array_s, gs_param_string) gs_param_s
   ((ps).data = (const byte *)(str),\
    (ps).size = strlen((const char *)(ps).data),\
    (ps).persistent = true)
-   
+
 #define param_string_from_transient_string(ps, str)\
   ((ps).data = (const byte *)(str),\
    (ps).size = strlen((const char *)(ps).data),\
