@@ -1,20 +1,22 @@
-/* Copyright (C) 1997, 1998, 1999, 2000 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 1997, 1998, 1999, 2000 artofcode LLC.  All rights reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; either version 2 of the License, or (at your
+  option) any later version.
+
+  This program is distributed in the hope that it will be useful, but
+  WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+  General Public License for more details.
+
+  You should have received a copy of the GNU General Public License along
+  with this program; if not, write to the Free Software Foundation, Inc.,
+  59 Temple Place, Suite 330, Boston, MA, 02111-1307.
+
 */
 
-/*$Id: gxdevice.h,v 1.4.6.2 2002/02/22 19:45:58 ray Exp $ */
+/*$Id: gxdevice.h,v 1.4.6.3.2.1 2003/01/17 00:49:03 giles Exp $ */
 /* Definitions for device implementors */
 
 #ifndef gxdevice_INCLUDED
@@ -52,9 +54,9 @@
 /* U.S. letter paper (8.5" x 11"). */
 #define DEFAULT_WIDTH_10THS_US_LETTER 85
 #define DEFAULT_HEIGHT_10THS_US_LETTER 110
-/* A4 paper (210mm x 297mm).  The dimensions are off by a few mm.... */
-#define DEFAULT_WIDTH_10THS_A4 83
-#define DEFAULT_HEIGHT_10THS_A4 117
+/* A4 paper (210mm x 297mm), we use 595pt x 842pt. */
+#define DEFAULT_WIDTH_10THS_A4 82.6389
+#define DEFAULT_HEIGHT_10THS_A4 116.9444
 /* Choose a default.  A4 may be set in the makefile. */
 #ifdef A4
 #  define DEFAULT_WIDTH_10THS DEFAULT_WIDTH_10THS_A4
@@ -76,6 +78,7 @@
  * but some compilers (the Ultrix 3.X pcc compiler and the HPUX compiler)
  * can't cast a computed float to an int.  That's why we specify
  * the page width and height in inches/10 instead of inches.
+ * This has been now been changed to use floats.
  *
  * Note that the macro is broken up so as to be usable for devices that
  * add further initialized state to the generic device.
@@ -94,8 +97,8 @@
  */
 #define std_device_part2_(width, height, x_dpi, y_dpi)\
 	{ gx_no_color_index, gx_no_color_index }, width, height,\
-	{ (((width) * 72.0 + 0.5) - 0.5) / (x_dpi)/*MediaSize[0]*/,\
-	  (((height) * 72.0 + 0.5) - 0.5) / (y_dpi)/*MediaSize[1]*/},\
+	{ (float)((((width) * 72.0 + 0.5) - 0.5) / (x_dpi))/*MediaSize[0]*/,\
+	  (float)((((height) * 72.0 + 0.5) - 0.5) / (y_dpi))/*MediaSize[1]*/},\
 	{ 0, 0, 0, 0 }/*ImagingBBox*/, 0/*ImagingBBox_set*/,\
 	{ x_dpi, y_dpi }/*HWResolution*/, { x_dpi, y_dpi }/*MarginsHWResolution*/
 
