@@ -1,20 +1,22 @@
-/* Copyright (C) 1989, 1995, 1996, 1997, 1998, 1999 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 1989, 1995, 1996, 1997, 1998, 1999 artofcode LLC.  All rights reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; either version 2 of the License, or (at your
+  option) any later version.
+
+  This program is distributed in the hope that it will be useful, but
+  WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+  General Public License for more details.
+
+  You should have received a copy of the GNU General Public License along
+  with this program; if not, write to the Free Software Foundation, Inc.,
+  59 Temple Place, Suite 330, Boston, MA, 02111-1307.
+
 */
 
-/*$Id: gxstroke.c,v 1.2.6.1 2002/02/22 19:45:59 ray Exp $ */
+/*$Id: gxstroke.c,v 1.2.6.2.2.1 2003/01/17 00:49:04 giles Exp $ */
 /* Path stroking procedures for Ghostscript library */
 #include "math_.h"
 #include "gx.h"
@@ -1195,13 +1197,18 @@ line_join_points(const gx_line_params * pgs_lp, pl_ptr plp, pl_ptr nplp,
 	double u1 = plp->e.cdelta.y, v1 = plp->e.cdelta.x;
 	double u2 = nplp->o.cdelta.y, v2 = nplp->o.cdelta.x;
 	double num, denom;
+	int code;
 
 	if (pmat) {
 	    gs_point pt;
 
-	    gs_distance_transform_inverse(v1, u1, pmat, &pt);
+	    code = gs_distance_transform_inverse(v1, u1, pmat, &pt);
+	    if (code < 0)
+		return code;
 	    v1 = pt.x, u1 = pt.y;
-	    gs_distance_transform_inverse(v2, u2, pmat, &pt);
+	    code = gs_distance_transform_inverse(v2, u2, pmat, &pt);
+	    if (code < 0)
+		return code;
 	    v2 = pt.x, u2 = pt.y;
 	    /*
 	     * We need to recompute ccw according to the
