@@ -1,20 +1,22 @@
-/* Copyright (C) 1989, 2000 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 1989, 2000 artofcode LLC.  All rights reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; either version 2 of the License, or (at your
+  option) any later version.
+
+  This program is distributed in the hope that it will be useful, but
+  WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+  General Public License for more details.
+
+  You should have received a copy of the GNU General Public License along
+  with this program; if not, write to the Free Software Foundation, Inc.,
+  59 Temple Place, Suite 330, Boston, MA, 02111-1307.
+
 */
 
-/*$Id: iscan.c,v 1.5.6.1 2002/02/22 19:45:59 ray Exp $ */
+/*$Id: iscan.c,v 1.5.6.5.2.1 2003/01/17 00:49:04 giles Exp $ */
 /* Token scanner for Ghostscript interpreter */
 #include "ghost.h"
 #include "memory_.h"
@@ -308,7 +310,7 @@ scan_comment(i_ctx_t *i_ctx_p, ref *pref, scanner_state *pstate,
 #ifdef DEBUG
 	if (gs_debug_c('%')) {
 	    dlprintf2("[%%%%%s%c]", sstr, (len >= 3 ? '+' : '-'));
-	    fwrite(base, 1, len, dstderr);
+	    debug_print_string(base, len);
 	    dputs("\n");
 	}
 #endif
@@ -326,7 +328,7 @@ scan_comment(i_ctx_t *i_ctx_p, ref *pref, scanner_state *pstate,
     else {
 	if (gs_debug_c('%')) {
 	    dlprintf2("[%% %s%c]", sstr, (len >= 2 ? '+' : '-'));
-	    fwrite(base, 1, len, dstderr);
+	    debug_print_string(base, len);
 	    dputs("\n");
 	}
     }
@@ -365,6 +367,7 @@ scan_string_token_options(i_ctx_t *i_ctx_p, ref * pstr, ref * pref,
 
     if (!r_has_attr(pstr, a_read))
 	return_error(e_invalidaccess);
+    s_init(s, NULL);
     sread_string(s, pstr->value.bytes, r_size(pstr));
     scanner_state_init_options(&state, options | SCAN_FROM_STRING);
     switch (code = scan_token(i_ctx_p, s, pref, &state)) {
@@ -742,16 +745,14 @@ scan_token(i_ctx_t *i_ctx_p, stream * s, ref * pref, scanner_state * pstate)
 #define comment_line da.buf
 		--sptr;
 		comment_line[1] = 0;
-		if (scan_comment_proc != NULL ||
-		    ((sptr == base || base[1] == '%') &&
-		     scan_dsc_proc != NULL)
-		    ) {		/* Could be an externally processable comment. */
+		{
+		    /* Could be an externally processable comment. */
 		    uint len = sptr + 1 - base;
+		    if (len > sizeof(comment_line))
+			len = sizeof(comment_line);
 
 		    memcpy(comment_line, base, len);
 		    daptr = comment_line + len;
-		} else {	/* Not a DSC comment. */
-		    daptr = comment_line + (max_comment_line + 1);
 		}
 		da.base = comment_line;
 		da.is_dynamic = false;
@@ -785,7 +786,7 @@ scan_token(i_ctx_t *i_ctx_p, stream * s, ref * pref, scanner_state * pstate)
 		    case char_EOL:
 		    case '\f':
 		      end_comment:
-			retcode = scan_comment(i_ctx_p, pref, &sstate,
+			retcode = scan_comment(i_ctx_p, myref, &sstate,
 					       comment_line, daptr, true);
 			if (retcode != 0)
 			    goto comment;
@@ -1119,9 +1120,12 @@ scan_token(i_ctx_t *i_ctx_p, stream * s, ref * pref, scanner_state * pstate)
     }
   sret:if (retcode < 0) {
 	scan_end_inline();
-	if (pstack != 0)
+	if (pstack != 0) {
+	    if (retcode == e_undefined)
+		*pref = *osp;	/* return undefined name as error token */
 	    ref_stack_pop(&o_stack,
 			  ref_stack_count(&o_stack) - (pdepth - 1));
+	}
 	return retcode;
     }
     /* If we are at the top level, return the object, */
