@@ -1,14 +1,25 @@
-/***************************************************************************
- *                                                                         *
- * Author : Jouk Jansen (joukj@hrem.stm.tudelft.nl)                        *
- *                                                                         *
- * Last revision : 22 August 2000                                          *
- *                                                                         *
- * Repair definitions of Xlib when compileing with /name=(as_is) on VMS    *
- *                                                                         *
- ***************************************************************************/
+/* Copyright (C) 2000 artofcode LLC.  All rights reserved.
+  
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; either version 2 of the License, or (at your
+  option) any later version.
 
-/* $Id: vms_x_fix.h,v 1.1.2.1 2002/04/10 09:33:26 giles Exp $ */
+  This program is distributed in the hope that it will be useful, but
+  WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+  General Public License for more details.
+
+  You should have received a copy of the GNU General Public License along
+  with this program; if not, write to the Free Software Foundation, Inc.,
+  59 Temple Place, Suite 330, Boston, MA, 02111-1307.
+
+*/
+
+/* $Id: vms_x_fix.h,v 1.1.2.2.2.1 2003/01/17 00:49:05 giles Exp $ */
+
+/* Repair definitions of Xlib when compileing with /name=(as_is) on VMS */
+/* contributed by Jouk Jansen (joukj@hrem.stm.tudelft.nl) */
 
 #ifndef VMS_X_FIX
 #define VMS_X_FIX
