@@ -1,20 +1,22 @@
-/* Copyright (C) 1993, 2000 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 1993, 2000 artofcode LLC.  All rights reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; either version 2 of the License, or (at your
+  option) any later version.
+
+  This program is distributed in the hope that it will be useful, but
+  WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+  General Public License for more details.
+
+  You should have received a copy of the GNU General Public License along
+  with this program; if not, write to the Free Software Foundation, Inc.,
+  59 Temple Place, Suite 330, Boston, MA, 02111-1307.
+
 */
 
-/*$Id: zchar1.c,v 1.7.2.1 2002/02/22 19:46:00 ray Exp $ */
+/*$Id: zchar1.c,v 1.7.2.2.2.1 2003/01/17 00:49:05 giles Exp $ */
 /* Type 1 character display operator */
 #include "memory_.h"
 #include "ghost.h"
@@ -199,6 +201,7 @@ charstring_execchar(i_ctx_t *i_ctx_p, int font_type_mask)
 	cxs.sbw[2] = 0;
 	cxs.sbw[3] = -penum->FontBBox_as_Metrics2.x; /* Sic! */
 	cxs.use_FontBBox_as_Metrics2 = true;
+	cxs.present = metricsSideBearingAndWidth;
     }
     /* Establish a current point. */
     code = gs_moveto(igs, 0.0, 0.0);
@@ -226,7 +229,8 @@ charstring_execchar(i_ctx_t *i_ctx_p, int font_type_mask)
 	const ref *opstr = op;
 	ref other_subr;
 
-	if (cxs.present == metricsSideBearingAndWidth) {
+	if (cxs.present == metricsSideBearingAndWidth
+	    && !cxs.use_FontBBox_as_Metrics2) {
 	    gs_point sbpt;
 
 	    sbpt.x = cxs.sbw[0], sbpt.y = cxs.sbw[1];
@@ -304,11 +308,48 @@ type1exec_bbox(i_ctx_t *i_ctx_p, gs_type1exec_state * pcxs,
     } else {
 	/* We have the width and bounding box: */
 	/* set up the cache device now. */
+        double w[2];
+        w[0] = pcxs->sbw[2], w[1] = pcxs->sbw[3];
+      
+	if (pcxs->use_FontBBox_as_Metrics2) {
+	   /* In this case, we have to calculate width for WMode=0. 
+	      pcxs->sbw[2, 3] is width for WMode=1.
+	      Normally, the width for WMode=0 is not used in WMode=1 
+	      rendering. However, if CDevProc is defined, 
+	      the width for WMode=0 is used. 
+	      Do the same as the case pcxs->present == metricsNone */
+	     double sbw[4];
+	     ref cnref;
+	     ref other_subr;
+	     int code;
+	     
+	    /* Since an OtherSubr callout might change osp, */
+	    /* save the character name now. */
+	    ref_assign(&cnref, op - 1);
+	    code = type1_continue_dispatch(i_ctx_p, pcxs, op, &other_subr, 4);
+	    op = osp;		/* OtherSubrs might change it */
+	    switch (code) {
+ 	     default:		/* code < 0 or done, error */
+		 return ((code < 0 ? code :
+			 gs_note_error(e_invalidfont)));
+	    case type1_result_callothersubr:	/* unknown OtherSubr */
+		return type1_call_OtherSubr(i_ctx_p, pcxs,
+					    bbox_getsbw_continue,
+					    &other_subr);
+	    case type1_result_sbw:	/* [h]sbw, done */
+		break;
+	    }
+	    type1_cis_get_metrics(pcis, sbw);
+	    w[0] = sbw[2], w[1] = sbw[3];
+	    /* Now actual width is available, I can calculate much 
+	       better side bearing for WMode 1 from the width. */
+	    pcxs->sbw[0] = w[0] / 2;
+	}
  	return zchar_set_cache(i_ctx_p, pbfont, op - 1,
 			       (pcxs->present == metricsSideBearingAndWidth
 			        && !pcxs->use_FontBBox_as_Metrics2 ?
 			        pcxs->sbw : NULL),
-			       pcxs->sbw + 2,
+			       w,
 			       &pcxs->char_bbox,
 			       bbox_finish_fill, bbox_finish_stroke, 
 			       (pcxs->use_FontBBox_as_Metrics2 ? pcxs->sbw : NULL));
@@ -505,6 +546,7 @@ bbox_draw(i_ctx_t *i_ctx_p, int (*draw)(P1(gs_state *)))
 	if (code < 0)
 	    return code;
 	cxs.present = code;
+	cxs.use_FontBBox_as_Metrics2 = false;
     }  else {
 	cxs.sbw[0] = penum->FontBBox_as_Metrics2.x / 2;
 	cxs.sbw[1] = penum->FontBBox_as_Metrics2.y;
@@ -516,7 +558,7 @@ bbox_draw(i_ctx_t *i_ctx_p, int (*draw)(P1(gs_state *)))
     code = type1_exec_init(&cxs.cis, penum, igs, pfont1);
     if (code < 0)
 	return code;
-    cxs.char_bbox = pfont1->FontBBox;
+    cxs.char_bbox = bbox;
     return type1exec_bbox(i_ctx_p, &cxs, pfont);
 }
 private int
@@ -717,6 +759,7 @@ nobbox_finish(i_ctx_t *i_ctx_p, gs_type1exec_state * pcxs)
     int code;
     gs_text_enum_t *penum = op_show_find(i_ctx_p);
     gs_font *pfont;
+    double w[2];
 
     if ((code = gs_pathbbox(igs, &pcxs->char_bbox)) < 0 ||
 	(code = font_param(op - 3, &pfont)) < 0
@@ -728,13 +771,24 @@ nobbox_finish(i_ctx_t *i_ctx_p, gs_type1exec_state * pcxs)
 	gs_font_base *const pbfont = (gs_font_base *) pfont;
 	gs_font_type1 *const pfont1 = (gs_font_type1 *) pfont;
 
-	if (pcxs->present == metricsNone) {
+	if (pcxs->present == metricsNone
+	    || pcxs->use_FontBBox_as_Metrics2) {
 	    gs_point endpt;
 
 	    if ((code = gs_currentpoint(igs, &endpt)) < 0)
 		return code;
-	    pcxs->sbw[2] = endpt.x, pcxs->sbw[3] = endpt.y;
+	    /* We will not use sbw[3, 4]. 
+	       If pcxs->use_FontBBox_as_Metrics2 is true,
+	       sbw[3, 4] are used as arguments(W1x, W1y) of setcachedevice2. 
+	       We will use w[0, 1] as W0x and W0y of setcachedevice2.
+	       W0 and W1 is differrent parameters. Don't confuse. */
+	    w[0] = endpt.x, w[1] = endpt.y;
 	    pcxs->present = metricsSideBearingAndWidth;
+	    if (pcxs->use_FontBBox_as_Metrics2) {
+	      /* Now actual width is available, I can calculate much 
+		 better side bearing for WMode 1 from the width. */
+	      pcxs->sbw[0] = w[0] / 2;
+	    }
 	}
 	/*
 	 * We only need to rebuild the path from scratch if we might
@@ -750,7 +804,7 @@ nobbox_finish(i_ctx_t *i_ctx_p, gs_type1exec_state * pcxs)
 	    return type1exec_bbox(i_ctx_p, pcxs, pfont);
 	}
 	return zchar_set_cache(i_ctx_p, pbfont, op, NULL,
-	                       pcxs->sbw + 2,
+	                       w,
 			       &pcxs->char_bbox,
 			       nobbox_fill, nobbox_stroke,
 			       (pcxs->use_FontBBox_as_Metrics2 ? pcxs->sbw : NULL));
