@@ -1,20 +1,22 @@
-/* Copyright (C) 1989, 2000, 2001 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 1989, 2000, 2001 artofcode LLC.  All rights reserved.
   
-  This software is provided AS-IS with no warranty, either express or
-  implied.
-  
-  This software is distributed under license and may not be copied,
-  modified or distributed except as expressly authorized under the terms
-  of the license contained in the file LICENSE in this distribution.
-  
-  For more information about licensing, please refer to
-  http://www.ghostscript.com/licensing/. For information on
-  commercial licensing, go to http://www.artifex.com/licensing/ or
-  contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-  San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; either version 2 of the License, or (at your
+  option) any later version.
+
+  This program is distributed in the hope that it will be useful, but
+  WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+  General Public License for more details.
+
+  You should have received a copy of the GNU General Public License along
+  with this program; if not, write to the Free Software Foundation, Inc.,
+  59 Temple Place, Suite 330, Boston, MA, 02111-1307.
+
 */
 
-/*$Id: zfile.c,v 1.11.2.3 2002/04/11 20:05:01 ray Exp $ */
+/*$Id: zfile.c,v 1.11.2.4.2.2 2003/05/17 14:28:33 ray Exp $ */
 /* Non-I/O file operators */
 #include "memory_.h"
 #include "string_.h"
@@ -51,7 +53,7 @@ extern_gx_io_device_table();
 extern const char iodev_dtype_stdio[];
 
 /* Forward references: file name parsing. */
-private int parse_file_name(P2(const ref * op, gs_parsed_file_name_t * pfn));
+private int parse_file_name(P3(const ref * op, gs_parsed_file_name_t * pfn, bool safemode));
 private int parse_real_file_name(P4(const ref * op,
 				    gs_parsed_file_name_t * pfn,
 				    gs_memory_t *mem, client_name_t cname));
@@ -149,22 +151,19 @@ check_file_permissions(i_ctx_t *i_ctx_p, const char *fname, int len,
     ref *permitlist = NULL;
     /* an empty string (first character == 0) if '\' character is */
     /* recognized as a file name separator as on DOS & Windows	  */
-    const char *filenamesep = gp_file_name_concat_string("\\", 1);
+    bool use_windows_pathsep = *gp_file_name_concat_string("\\", 1) == '\0';
+    bool fname_bare = !gp_pathstring_not_bare(fname, len);
+    const char *sep_string = NULL;
+    int cwd_len = 0, sep_len = 0;
 
-    /*
-     * Check here for the %pipe device which is illegal when
-     * LockFilePermissions is true. In the future we might want to allow
-     * the %pipe device to be included on the PermitFile... paths, but
-     * for now it is simply disallowed.
-     */
-    if (i_ctx_p->LockFilePermissions &&
-               string_match( (const unsigned char*) fname, len,
-			     (const unsigned char*) "%pipe*", 5, NULL)
-       ) {
-	return e_invalidfileaccess;
-    }
     if (dict_find_string(&(i_ctx_p->userparams), permitgroup, &permitlist) <= 0)
         return 0;	/* if Permissions not found, just allow access */
+    if (fname_bare) {
+	cwd_len = strlen(gp_current_directory_name);
+	sep_string = gp_file_name_concat_string(gp_current_directory_name, cwd_len);
+	sep_len = strlen(sep_string);
+    }
+
     for (i=0; i<r_size(permitlist); i++) {
         ref permitstring;
 	const string_match_params win_filename_params = {
@@ -177,7 +176,7 @@ check_file_permissions(i_ctx_t *i_ctx_p, const char *fname, int len,
 	    break;	/* any problem, just fail */
         if (string_match( (const unsigned char*) fname, len,
 			  permitstring.value.bytes, r_size(&permitstring), 
-		filenamesep[0] == 0 ? &win_filename_params : NULL)
+		use_windows_pathsep ? &win_filename_params : NULL)
 	   ) {
 	    /*
 	     * We can't know where we will get to if we reference the parent
@@ -191,6 +190,18 @@ check_file_permissions(i_ctx_t *i_ctx_p, const char *fname, int len,
 	    else
 		return 0;		/* success */
 	}
+	    /* fname is a bare name meaning it will default to the current
+	     * directory. Check to see if the current permitted path is the
+             * platform's current directory, and if so, succeed.
+	     */
+	if (fname_bare && 
+	    (r_size(&permitstring) >= cwd_len + sep_len + 1) &&
+		(bytes_compare(permitstring.value.bytes, cwd_len,
+			(const byte *)gp_current_directory_name, cwd_len) == 0) &&
+		(bytes_compare(permitstring.value.bytes+cwd_len, sep_len,
+			(const byte *)sep_string, sep_len) == 0) &&
+		(permitstring.value.bytes[cwd_len+sep_len] == '*'))
+	    return 0;	/* permitstring was CWD + sep + * */
     }
     /* not found */
     return e_invalidfileaccess;
@@ -208,7 +219,7 @@ zfile(i_ctx_t *i_ctx_p)
 
     if (code < 0)
 	return code;
-    code = parse_file_name(op - 1, &pname);
+    code = parse_file_name(op - 1, &pname, i_ctx_p->LockFilePermissions);
     if (code < 0)
 	return code;
 	/*
@@ -386,15 +397,26 @@ zrenamefile(i_ctx_t *i_ctx_p)
     pname2.fname = 0;
     code = parse_real_file_name(op, &pname2, imemory, "renamefile(to)");
     if (code >= 0) {
+	if (pname1.iodev != pname2.iodev ) {
+	    if (pname1.iodev == iodev_default)
+		pname1.iodev = pname2.iodev;
+	    if (pname2.iodev == iodev_default)
+		pname2.iodev = pname1.iodev;
+	}
         if (pname1.iodev != pname2.iodev ||
-	      (check_file_permissions(i_ctx_p, pname1.fname, pname1.len,
+	    (pname1.iodev == iodev_default &&
+		/*
+		 * We require FileControl permissions on the source path
+		 * unless it is a temporary file. Also, we require FileControl
+		 * and FileWriting permissions to the destination file/path.
+		 */
+	      ((check_file_permissions(i_ctx_p, pname1.fname, pname1.len,
 	      				"PermitFileControl") < 0 &&
-	          !file_is_tempfile(i_ctx_p, op - 1) < 0) ||
-	      check_file_permissions(i_ctx_p, pname2.fname, pname2.len,
+	          !file_is_tempfile(i_ctx_p, op - 1)) ||
+	      (check_file_permissions(i_ctx_p, pname2.fname, pname2.len,
 	      				"PermitFileControl") < 0 ||
 	      check_file_permissions(i_ctx_p, pname2.fname, pname2.len,
-	      				"PermitFileWriting") < 0 ) {
-	    /* add a check to permit pname1 to be a tempfile */
+	      				"PermitFileWriting") < 0 )))) {
 	    code = gs_note_error(e_invalidfileaccess);
 	} else {
 	    code = (*pname1.iodev->procs.rename_file)(pname1.iodev,
@@ -429,7 +451,7 @@ zstatus(i_ctx_t *i_ctx_p)
 	    {
 		gs_parsed_file_name_t pname;
 		struct stat fstat;
-		int code = parse_file_name(op, &pname);
+		int code = parse_file_name(op, &pname, i_ctx_p->LockFilePermissions);
 
 		if (code < 0)
 		    return code;
@@ -565,7 +587,7 @@ zlibfile(i_ctx_t *i_ctx_p)
     stream *s;
 
     check_ostack(2);
-    code = parse_file_name(op, &pname);
+    code = parse_file_name(op, &pname, i_ctx_p->LockFilePermissions);
     if (code < 0)
 	return code;
     if (pname.iodev == NULL)
@@ -598,9 +620,9 @@ zlibfile(i_ctx_t *i_ctx_p)
 	    /* Check to see if this file is allowed */
 	    /* Possibly we should allow access to parent directories if */
 	    /* PermitFileReading includes (*), but this is unlikely if  */
-	    /* we are locked. */
+	    /* we are locked (and we checked LockFilePerminssions above).*/
 	    if (gp_file_name_references_parent(pname.fname, pname.len) ||
-		(gp_file_name_is_absolute(pname.fname, pname.len) &&
+		(gp_pathstring_not_bare(pname.fname, pname.len) &&
 		check_file_permissions(i_ctx_p, pname.fname, pname.len,
 					"PermitFileReading") < 0)
 	       ) 
@@ -617,7 +639,7 @@ zlibfile(i_ctx_t *i_ctx_p)
 	    }
 	}
 	if (code < 0) {
-	    if (code == e_VMerror)
+	    if (code == e_VMerror || code == e_invalidfileaccess)
 		return code;
 	    push(1);
 	    make_false(op);
@@ -663,7 +685,7 @@ ztempfile(i_ctx_t *i_ctx_p)
     }
     if (i_ctx_p->LockFilePermissions) 
         if (gp_file_name_references_parent(pstr, strlen(pstr)) ||
-	    (gp_file_name_is_absolute(pstr, strlen(pstr)) &&
+	    (gp_pathstring_not_bare(pstr, strlen(pstr)) &&
 	      check_file_permissions(i_ctx_p, pstr, strlen(pstr),
 	      				"PermitFileWriting") < 0 )
 	)
@@ -694,6 +716,17 @@ ztempfile(i_ctx_t *i_ctx_p)
     return code;
 }
 
+/* <string> .pathstring_not_bare <bool> */
+private int
+zpathstring_not_bare(i_ctx_t *i_ctx_p)
+{
+    os_ptr op = osp;
+    check_type(*op, t_string);
+    make_bool(op, gp_pathstring_not_bare((char *)op->value.bytes, r_size(op)));
+    return 0;
+}
+
+
 /* ------ Initialization procedure ------ */
 
 const op_def zfile_op_defs[] =
@@ -709,6 +742,7 @@ const op_def zfile_op_defs[] =
     {"2renamefile", zrenamefile},
     {"1status", zstatus},
     {"2.tempfile", ztempfile},
+    {"2.pathstring_not_bare", zpathstring_not_bare},
 		/* Internal operators */
     {"0%file_continue", file_continue},
     {"0%execfile_finish", execfile_finish},
@@ -720,11 +754,24 @@ const op_def zfile_op_defs[] =
 /* Parse a file name into device and individual name. */
 /* See gsfname.c for details. */
 private int
-parse_file_name(const ref * op, gs_parsed_file_name_t * pfn)
+parse_file_name(const ref * op, gs_parsed_file_name_t * pfn, bool safemode)
 {
+    int code;
+
     check_read_type(*op, t_string);
-    return gs_parse_file_name(pfn, (const char *)op->value.const_bytes,
+    code = gs_parse_file_name(pfn, (const char *)op->value.const_bytes,
 			      r_size(op));
+    if (code < 0)
+	return code;
+    /*
+     * Check here for the %pipe device which is illegal when
+     * LockFilePermissions is true. In the future we might want to allow
+     * the %pipe device to be included on the PermitFile... paths, but
+     * for now it is simply disallowed.
+     */
+    if (pfn->iodev && safemode && strcmp(pfn->iodev->dname, "%pipe%") == 0)
+	return e_invalidfileaccess;
+    return code;
 }
 
 /* Parse a real (non-device) file name and convert to a C string. */
@@ -848,7 +895,7 @@ lib_file_fopen(gx_io_device * iodev, const char *bname,
 
     strcpy(fmode, "r");
     strcat(fmode, gp_fmode_binary_suffix);
-    if (gp_file_name_is_absolute(bname, len))
+    if (gp_pathstring_not_bare(bname, len))
 	return (*iodev->procs.fopen)(iodev, bname, fmode, pfile,
 				     rfname, rnamelen);
     /* Go through the list of search paths */
