@@ -1,23 +1,22 @@
-/* Copyright (C) 1989, 1995, 1997, 1998, 1999 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 1989, 1995, 1997, 1998, 1999 artofcode LLC.  All rights reserved.
   
-  This file is part of GNU Ghostscript.
-  
-  GNU Ghostscript is distributed in the hope that it will be useful, but
-  WITHOUT ANY WARRANTY.  No author or distributor accepts responsibility
-  to anyone for the consequences of using it or for whether it serves any
-  particular purpose or works at all, unless he says so in writing.  Refer
-  to the GNU General Public License for full details.
-  
-  Everyone is granted permission to copy, modify and redistribute GNU
-  Ghostscript, but only under the conditions described in the GNU General
-  Public License.  A copy of this license is supposed to have been given
-  to you along with GNU Ghostscript so you can know your rights and
-  responsibilities.  It should be in a file named COPYING.  Among other
-  things, the copyright notice and this notice must be preserved on all
-  copies.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; either version 2 of the License, or (at your
+  option) any later version.
+
+  This program is distributed in the hope that it will be useful, but
+  WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+  General Public License for more details.
+
+  You should have received a copy of the GNU General Public License along
+  with this program; if not, write to the Free Software Foundation, Inc.,
+  59 Temple Place, Suite 330, Boston, MA, 02111-1307.
+
 */
 
-/*$RCSfile: zvmem.c,v $ $Revision: 1.2.2.2 $ */
+/*$Id: zvmem.c,v 1.3.2.2.2.1 2003/01/17 00:49:06 giles Exp $ */
 /* "Virtual memory" operators */
 #include "ghost.h"
 #include "gsstruct.h"
@@ -37,10 +36,10 @@
 
 /* Define whether we validate memory before/after save/restore. */
 /* Note that we only actually do this if DEBUG is set and -Z? is selected. */
-private bool I_VALIDATE_BEFORE_SAVE = true;
-private bool I_VALIDATE_AFTER_SAVE = true;
-private bool I_VALIDATE_BEFORE_RESTORE = true;
-private bool I_VALIDATE_AFTER_RESTORE = true;
+private const bool I_VALIDATE_BEFORE_SAVE = true;
+private const bool I_VALIDATE_AFTER_SAVE = true;
+private const bool I_VALIDATE_BEFORE_RESTORE = true;
+private const bool I_VALIDATE_AFTER_RESTORE = true;
 
 /* 'Save' structure */
 typedef struct vm_save_s vm_save_t;
