@@ -4,20 +4,21 @@ exec tclsh "$0" "$@"
 
 #    Copyright (C) 2000 Aladdin Enterprises.  All rights reserved.
 # 
-# This software is provided AS-IS with no warranty, either express or
-# implied.
-# 
-# This software is distributed under license and may not be copied,
-# modified or distributed except as expressly authorized under the terms
-# of the license contained in the file LICENSE in this distribution.
-# 
-# For more information about licensing, please refer to
-# http://www.ghostscript.com/licensing/. For information on
-# commercial licensing, go to http://www.artifex.com/licensing/ or
-# contact Artifex Software, Inc., 101 Lucas Valley Road #110,
-# San Rafael, CA  94903, U.S.A., +1(415)492-9861.
+# This program is free software; you can redistribute it and/or modify it
+# under the terms of the GNU General Public License as published by the
+# Free Software Foundation; either version 2 of the License, or (at your
+# option) any later version.
+#
+# This program is distributed in the hope that it will be useful, but
+# WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
+# Public License for more details.
+#
+# You should have received a copy of the GNU General Public License along
+# with this program; if not, write to the Free Software Foundation, Inc.,
+# 59 Temple Place, Suite 330, Boston, MA, 02111-1307.
 
-# $Id: 3way.tcl,v 1.3.6.1 2002/02/22 19:46:03 ray Exp $
+# $Id: 3way.tcl,v 1.3.6.1.2.1 2003/04/12 14:02:39 giles Exp $
 
 # This file is supposed to help with 3-way merges.
 # Currently there is only one function:
@@ -42,8 +43,8 @@ proc filesame {f1 f2} {
 	} {
 	    if {[regexp {Copyright} $a]} {
 		if {[regexp {Copyright} $c]} {continue}
-	    } elseif {[regexp {\$Id: 3way.tcl,v 1.3.6.1 2002/02/22 19:46:03 ray Exp $a]} {
-		if {[regexp {\$Id: 3way.tcl,v 1.3.6.1 2002/02/22 19:46:03 ray Exp $c]} {continue}
+	    } elseif {[regexp {\$Id: 3way.tcl,v 1.3.6.1.2.1 2003/04/12 14:02:39 giles Exp $a]} {
+		if {[regexp {\$Id: 3way.tcl,v 1.3.6.1.2.1 2003/04/12 14:02:39 giles Exp $c]} {continue}
 	    }
 	}
 	set same 0
